"""
Income Threshold   Rate     PAYE
14000               10.5    1470
48000               17.5    5950
70000               30      6600
70000+              33      x
"""
from nzpaye.exception import ArgumentError

summary = {}
INCOME_THRESHOLD_A = 14000
INCOME_THRESHOLD_B = 48000
INCOME_THRESHOLD_C = 70000
HOURS_IN_A_WEEK = 40
WEEKS_IN_A_YEAR = 52
GST_RATE = 0.15

def validate_params(hourly_rate, hours_worked, witholding_tax):
    if hours_worked <= 0:
        raise ArgumentError("Enter a valid value for hours worked")
    if hourly_rate <= 0 :
        raise ArgumentError("Enter a valid value for hourly rate")
    if not(witholding_tax == 10 or witholding_tax == 20):
        raise ArgumentError("Enter a valid value for witholding tax")
    return True

def income_summary(hourly_rate, hours_worked, witholding_tax=10):
    # assuming working each week for 40 hours each year
    validate_params(hourly_rate, hours_worked, witholding_tax)
    yearly_income = hourly_rate * HOURS_IN_A_WEEK * WEEKS_IN_A_YEAR
    yearly_paye = calculate_yearly_paye(yearly_income)

    hourly_paye = yearly_paye / (WEEKS_IN_A_YEAR * HOURS_IN_A_WEEK)

    #assuming gst is 15%
    hourly_gst = GST_RATE * hourly_rate

    total_income = hours_worked * hourly_rate
    total_paye = hours_worked * hourly_paye
    less_witholding_tax = (witholding_tax/100) * total_income
    plus_gst = hours_worked * hourly_gst
    paid_to_account = total_income - less_witholding_tax + plus_gst
    remaining_paye_to_be_paid = (hours_worked * hourly_paye) - less_witholding_tax
    remaining_gst_and_paye_to_be_paid = plus_gst + remaining_paye_to_be_paid
    disposable_income = paid_to_account - remaining_gst_and_paye_to_be_paid

    summary['hours_worked'] = str(hours_worked)
    summary['total_income'] = format(total_income, '.2f')
    summary['total_paye'] = format(total_paye, '.2f')
    summary['less_witholding_tax'] = format(less_witholding_tax, '.2f')
    summary['plus_gst'] = format(plus_gst, '.2f')
    summary['paid_to_account'] = format(paid_to_account, '.2f')
    summary['remaining_paye_to_be_paid'] = format(remaining_paye_to_be_paid, '.2f')
    summary['remaining_gst_and_paye_to_be_paid'] = format(remaining_gst_and_paye_to_be_paid, '.2f')
    summary['disposable_income'] = format(disposable_income, '.2f')

    return summary


def calculate_yearly_paye(yearly_income):
    yearly_paye = 0
    if yearly_income > 0 and yearly_income <= INCOME_THRESHOLD_A:
        yearly_paye = yearly_paye + (0.105 * yearly_income)

    if yearly_income > INCOME_THRESHOLD_A and yearly_income <= INCOME_THRESHOLD_B:
        yearly_paye = yearly_paye + 1470
        yearly_paye = yearly_paye + (0.175 * (yearly_income - INCOME_THRESHOLD_A))

    yearly_paye = check_and_apply_ietc(yearly_income, yearly_paye)

    if yearly_income > INCOME_THRESHOLD_B and yearly_income <= INCOME_THRESHOLD_C:
        yearly_paye = yearly_paye + 1470 + 5950
        yearly_paye = yearly_paye + (0.3 * (yearly_income - INCOME_THRESHOLD_B))

    if yearly_income > INCOME_THRESHOLD_C:
        yearly_paye = yearly_paye + 1470 + 5950 + 6600
        yearly_paye = yearly_paye + (0.33 * (yearly_income - INCOME_THRESHOLD_C))

    return round(yearly_paye, 2)


def check_and_apply_ietc(yearly_income, yearly_paye):
    if yearly_income >= 24000 and yearly_income <= 44000:
        yearly_paye = yearly_paye - 520
    if yearly_income > 44000 and yearly_income <= 48000:
        yearly_paye = yearly_paye - (520 - (0.13 * (yearly_income - 44000)))
    return yearly_paye


import logging
import logging.config
import socket

HOSTNAME = socket.gethostname()

def setup_logging(loglevel, logfile=None):
    loglevel = loglevel.upper()

    LOGGING_CONFIG = {
        "version"                 : 1,
        "disable_existing_loggers": False,
        "formatters"              : {
            "default": {
                "format": "[%(asctime)s] {0}/%(levelname)s/%(name)s: %(message)s".format(HOSTNAME),
            },
            "plain"  : {
                "format": "%(message)s",
            },
        },
        "handlers"                : {
            "console"      : {
                "class"    : "logging.StreamHandler",
                "formatter": "default",
            },
            "console_plain": {
                "class"    : "logging.StreamHandler",
                "formatter": "plain",
            },
        },
        "loggers"                 : {
            "paye"             : {
                "handlers" : ["console"],
                "level"    : loglevel,
                "propagate": False,
            },
        },
        "root"                    : {
            "handlers": ["console"],
            "level"   : loglevel,
        },
    }
    logging.config.dictConfig(LOGGING_CONFIG)




from nzpaye.main import paye_summary
from nzpaye import argument_parser


def main():
    parser = argument_parser.get_parser()
    options = parser.parse_args()
    paye_summary(options)

if __name__ == "__main__":
    main()


from setuptools import setup

setup(
    name='nzpaye',
    version='0.1.1',
    description='NZ Paye Summary',
    long_description="""Calculate the NZ Paye Summary based on the hourly rate and the number of hours worked.""",
    url='https://github.com/anuj-ssharma/NZPaye',
    author='Anuj Sharma',
    author_email='anuj576@gmail.com',
    license='MIT',
    packages=['nzpaye'],
    install_requires=['tabulate==0.8.7'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Financial and Insurance Industry',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.8',
    ],
    test_suite='nzpaye.test',
    tests_require=[
            'mock'
    ],
    entry_points={
        'console_scripts': [
            'nzpaye = nzpaye.__main__:main',
        ]
    }
)



from tabulate import tabulate
from nzpaye.paye import income_summary
import sys
from nzpaye import argument_parser
import logging
from nzpaye.log import setup_logging

setup_logging("ERROR")
logger = logging.getLogger("paye")

def validate_argument_values(options):
    try:
        if not (options.witholding_tax == 10.0 or options.witholding_tax == 20.0):
            logger.error("Witholding tax can be 10 or 20 ")
            sys.exit(1)
        if options.hours_worked <= 0:
            logger.error("Hours worked should be greater than 0")
            sys.exit(1)
        if options.hourly_rate <= 0:
            logger.error("Hourly rate should be greater than 0")
            sys.exit(1)
    except ValueError as e:
        logger.error(e)
        sys.exit(1)
    return True

def paye_summary(options):
    try:
        validate_argument_values(options)
        summary = income_summary(options.hourly_rate, options.hours_worked, options.witholding_tax)
        print(tabulate([list(summary.values())],
                       headers=["Hours Worked", "Total Income", "Total Paye", "Less Witholding Tax", "Plus GST", "Paid to Account",
                                "Remaining PAYE", "Remaining GST + PAYE", "Disposable Income"], tablefmt='pretty'))
    except Exception as e:
        logger.error("Oops something went wrong. ")
        logger.error(str(e))
        sys.exit(1)




import argparse

def setup_parser_arguments(parser):
    parser.add_argument(
           '--hourly-rate',
            type=float,
            dest='hourly_rate',
            required=True,
            help="Enter the hourly rate"
    )
    parser.add_argument(
            '--hours-worked',
            type=float,
            dest='hours_worked',
            required=True,
            help='Enter the number of hours worked'
    )
    parser.add_argument(
            '--wht',
            type=float,
            dest='witholding_tax',
            default=10.0,
            help='Enter your witholding tax i.e. 10 or 20. Default value is 10.'
    )
    return parser

def get_parser():
    parser = argparse.ArgumentParser(description="Income Summary calculator")
    setup_parser_arguments(parser)
    return parser